@@ -65,8 +65,7 @@
 #' @param repo URL of the repository.
 #' @param tarball The URL or local path to a tarball with a CWB indexed corpus.
 #' @param doi The DOI (Digital Object Identifier) of a corpus deposited at
-#'   Zenodo, presented as a hyperlink. (For testing purposes, use
-#'   ("https://doi.org/10.5281/zenodo.3748858".)
+#'   Zenodo (e.g. "10.5281/zenodo.3748858".)
 #' @param lib Directory for R packages, defaults to \code{.libPaths()[1]}.
 #' @param verbose Logical, whether to be verbose.
 #' @param ask A \code{logical} value, whether to ask for user input when
@@ -92,11 +91,19 @@
 #' @importFrom jsonlite fromJSON
 #' @importFrom utils menu
 #' @importFrom stringi stri_enc_mark
+#' @importFrom tools file_path_sans_ext
+#' @importFrom zen4R ZenodoManager
+#' @importFrom cli cli_rule cli_alert_success cli_process_start cli_process_done
+#'   cli_alert_info col_cyan cli_alert_danger cli_text col_blue col_red cli_alert_warning
 #' @rdname corpus_utils
 #' @export corpus_install
 corpus_install <- function(pkg = NULL, repo = "https://PolMine.github.io/drat/", tarball = NULL, doi = NULL, lib = .libPaths()[1], registry_dir = cwbtools::cwb_registry_dir(), corpus_dir = cwb_corpus_dir(registry_dir), ask = interactive(), verbose = TRUE, user = NULL, password = NULL, ...){
+  
+  modify_renviron <- FALSE
+  
   if (is.null(tarball)){
     if (isFALSE(is.null(pkg))){
+      # A pkg is stated 
       if (!pkg %in% utils::available.packages(contriburl = utils::contrib.url(repos = repo))) {
         stop(sprintf("package '%s' not available at repo '%s'", pkg, repo))
       }
@@ -121,16 +128,20 @@ corpus_install <- function(pkg = NULL, repo = "https://PolMine.github.io/drat/",
       # Turn DOI into tarball
       
       if (is.null(doi)) stop("If argument 'tarball' is NULL, either argument 'pkg' or argument 'doi' are required.")
+      if (grepl("https://doi.org/", doi)) doi <- gsub("https://doi.org/", "", doi)
       if (isFALSE(grepl("^.*?10\\.5281/zenodo\\.\\d+$", doi))){
         stop("Argument 'doi' is expected to offer a DOI (Digital Object Identifier) that refers to data",
              "hosted with zenodo, i.e. starting with 10.5281/zenodo.")
       }
-      zenodo_info <- .zenodo_info(doi = doi)
-      tarball <- grep(
-        "^.*?_v\\d+\\.\\d+\\.\\d+\\.tar\\.gz$",
-        zenodo_info[["files"]][["links"]][["self"]],
-        value = TRUE
-      )
+      
+      if (verbose) cli_rule("Resolve DOI")
+      if (verbose) cli_process_start("get metadata for the Zenodo record referred to by DOI")
+      zenodo_record <- ZenodoManager$new()$getRecordByDOI(doi = doi)
+      if (verbose) cli_process_done()
+      
+      zenodo_files <- sapply(zenodo_record[["files"]], function(x) x[["links"]][["download"]])
+      tarball <- grep("^.*?_(v|)\\d+\\.\\d+\\.\\d+\\.tar\\.gz$", zenodo_files, value = TRUE)
+      
       if (length(tarball) > 1L && isTRUE(ask)){
         userchoice <- utils::menu(
           choices = basename(tarball),
@@ -138,23 +149,27 @@ corpus_install <- function(pkg = NULL, repo = "https://PolMine.github.io/drat/",
         )
         tarball <- tarball[userchoice]
       }
+      if (verbose) cli_alert_info(sprintf("tarball to be downloaded: %s", col_blue(basename(tarball))))
     }
   }
   
   # Create CWB directory structure if necessary --------------
   
   cwb_dirs <- cwb_directories(registry_dir = registry_dir, corpus_dir = corpus_dir)
-  if (any(is.null(cwb_dirs))) cwb_dirs <- create_cwb_directories() # will trigger interactive dialogue
+  if (any(is.null(cwb_dirs))){
+    cwb_dirs <- create_cwb_directories() # will trigger interactive dialogue
+    modify_renviron <- TRUE
+  }
 
   if (length(tarball) == 1L){
     
     # Ask user before overwriting existing corpus ----------------
-    corpus <- gsub("^(.*?)(_v\\d+\\.\\d+\\.\\d+|)\\.tar\\.gz$", "\\1", basename(tarball))
-    version <- if (exists("zenodo_info")){
-      zenodo_info[["metadata"]][["version"]]
+    corpus <- gsub("^(.*?)_(v|)(\\d+\\.\\d+\\.\\d+|)\\.tar\\.gz$", "\\1", basename(tarball))
+    version <- if (exists("zenodo_record")){
+      zenodo_record[["metadata"]][["version"]]
     } else {
-      if (grepl("^.*?_v\\d+\\.\\d+\\.\\d+\\.tar\\.gz$", basename(tarball))){
-        gsub("^.*?_(v\\d+\\.\\d+\\.\\d+)\\.tar\\.gz$", "\\1", basename(tarball))
+      if (grepl("^.*?\\d+\\.\\d+\\.\\d+\\.tar\\.gz$", basename(tarball))){
+        gsub("^.*?_(v|)(\\d+\\.\\d+\\.\\d+)\\.tar\\.gz$", "v\\2", basename(tarball))
       } else {
         "unknown"
       }
@@ -167,44 +182,31 @@ corpus_install <- function(pkg = NULL, repo = "https://PolMine.github.io/drat/",
         "unknown"
       }
       if (ask){
+        cli_rule("Remove existing corpus")
         if (version_old == version){
-          userinput <- menu(
-            choices = c("Yes / continue", "No / abort"), 
-            title = sprintf(
+          cli_text(sprintf(
               paste(
                 "Local %s version and the version of %s to be downloaded are identical (%s).",
-                "Are you sure you want to proceed and to replace the local corpus by a fresh download?"
+                "If you proceed, the local corpus will be replaced by a fresh download."
               ),
-              toupper(corpus), toupper(corpus), version
+              col_red(toupper(corpus)), col_red(toupper(corpus)), col_blue(version)
             )
           )
-          if (userinput != 1L){
-            stop(sprintf("Aborting - existing version of %s remains unchanged. ", toupper(corpus)))
-          }
         } else {
-          userinput <- menu(
-            choices = c("Yes / continue", "No / abort"), 
-            title = sprintf(
-              "Corpus %s (version: %s) is already installed. Do you want to replace it by version %s?",
+          cli_text(
+            sprintf(
+              "Corpus %s (version: %s) is already installed. If you proceed, it will be replaced by version %s.",
               toupper(corpus), version_old, version
             )
           )
-          if (userinput != 1L){
-            stop(
-              sprintf(paste("Aborting - existing version of %s remains unchanged. ",
-                            "If you want to keep the existing version (%s), rename corpus using cwbtools::corpus_rename(), ",
-                            "and call corpus_install() again."
-              ),
-              toupper(corpus), version
-              )
-            ) 
-          }
         }
       }
       corpus_remove(corpus = toupper(corpus), registry_dir = cwb_dirs[["registry_dir"]], ask = ask)
     }
     
     # Now download corpus -------------------
+    cat_rule("Download corpus tarball")
+    if (verbose) cli_alert_info(sprintf("download corpus tarball {col_cyan('%s')}", basename(tarball)))
     cwbtools_tmpdir <- file.path(normalizePath(tempdir(), winslash = "/"), "cwbtools_tmpdir", fsep = "/")
     if (file.exists(cwbtools_tmpdir)) unlink(cwbtools_tmpdir, recursive = TRUE)
     dir.create(cwbtools_tmpdir)
@@ -243,16 +245,20 @@ corpus_install <- function(pkg = NULL, repo = "https://PolMine.github.io/drat/",
       if (!file.exists(tarball)) stop(sprintf("tarball '%s' not found locally", tarball))
       file.copy(from = tarball, to = corpus_tarball)
     }
-    if (.Platform$OS.type == "windows" && stri_enc_mark(corpus_tarball) != "ASCII"){
-      corpus_tarball <- utils::shortPathName(corpus_tarball)
-    }
-    if (.Platform$OS.type == "windows" && stri_enc_mark(cwbtools_tmpdir) != "ASCII"){
-      cwbtools_tmpdir <- utils::shortPathName(cwbtools_tmpdir)
-    }
-    if (verbose) message("... extracting tarball")
+    if (verbose) cli_alert_success(sprintf("download corpus tarball {col_cyan('%s')} ... done", basename(tarball)))
     
+    if (.Platform$OS.type == "windows"){
+      if (stri_enc_mark(corpus_tarball) != "ASCII") corpus_tarball <- utils::shortPathName(corpus_tarball)
+      if (stri_enc_mark(cwbtools_tmpdir) != "ASCII") cwbtools_tmpdir <- utils::shortPathName(cwbtools_tmpdir)
+    }
+
+    if (verbose) cli_process_start("extract tarball")
     untar(tarfile = corpus_tarball, exdir = cwbtools_tmpdir)
+    if (verbose) cli_process_done()
+
+    if (verbose) cli_process_start("remove tarball")
     unlink(corpus_tarball)
+    if (verbose) cli_process_done()
     
     tmp_registry_dir <- file.path(normalizePath(cwbtools_tmpdir, winslash = "/"), "registry", fsep = "/")
     tmp_data_dir <- file.path(normalizePath(cwbtools_tmpdir, winslash = "/"), "indexed_corpora", fsep = "/")
@@ -263,6 +269,7 @@ corpus_install <- function(pkg = NULL, repo = "https://PolMine.github.io/drat/",
       registry_data <- registry_file_parse(corpus = corpus, registry_dir = tmp_registry_dir)
       
       tmp_home_dir <- file.path(tmp_data_dir, tolower(corpus), fsep = "/")
+      
       if (.Platform$OS.type == "windows" && stri_enc_mark(tmp_home_dir) != "ASCII")
         tmp_home_dir <- utils::shortPathName(tmp_home_dir)
       registry_data[["home"]] <- tmp_home_dir
@@ -291,7 +298,7 @@ corpus_install <- function(pkg = NULL, repo = "https://PolMine.github.io/drat/",
           corpus = corpus,
           registry_dir = tmp_registry_dir, 
           data_dir = tmp_home_dir, # the temporary place
-          registry_dir_new = registry_dir, 
+          registry_dir_new = cwb_dirs[["registry_dir"]], 
           data_dir_new = data_dir_target # final location
         )
       }
@@ -299,17 +306,22 @@ corpus_install <- function(pkg = NULL, repo = "https://PolMine.github.io/drat/",
       # Resetting registry and copying registry file to polmineR's temporary registry, 
       # if polmineR is loaded necessary
       if (isNamespaceLoaded("polmineR")){
-        file.copy(
-          from = file.path(cwb_dirs[["registry_dir"]], tolower(corpus)),
-          to = file.path(polmineR::registry(), tolower(corpus)),
-          overwrite = TRUE
-        )
-        polmineR::registry_reset(registryDir = polmineR::registry())
+        destfile <- file.path(cwb_dirs[["registry_dir"]], tolower(corpus))
+        targetfile <- file.path(polmineR::registry(), tolower(corpus))
+        if (destfile != targetfile){
+          file.copy(from = destfile, to = targetfile, overwrite = TRUE)
+        }
+        polmineR::registry_reset(registryDir = polmineR::registry(), verbose = FALSE)
       }
 
     }
     unlink(cwbtools_tmpdir, recursive = TRUE)
-    return(invisible(NULL))
+    
+    # Check whether newly installed corpoa can be loaded
+    # for (corpus_id in corpora){
+    #   corpus_testload(corpus = corpus_id, registry_dir = cwb_dirs[["registry_dir"]])
+    # }
+    
   } else {
     for (tarfile in tarball){
       corpus_install(
@@ -324,6 +336,39 @@ corpus_install <- function(pkg = NULL, repo = "https://PolMine.github.io/drat/",
       )
     }
   }
+  if (isTRUE(modify_renviron)){
+    cli_alert_info(
+      paste(
+        sprintf("You have created the registry directory {.path %s} anew.", cwb_dirs[["registry_dir"]]),
+        "The environment variable {.envvar CORPUS_REGISTRY} needs to refer to this directory to make the newly installed corpus available.",
+        sprintf("You can call {.code Sys.getenv(CORPUS_REGISTRY=\"%s\")} whenever the environment variable needs to be set.", cwb_dirs[["registry_dir"]]),
+        "It is necessary to set the CORPUS_REGISTRY environment variable before loading polmineR.",
+        sprintf("To make this setting persist across sessions, you can add the line {.code CORPUS_REGISTRY=%s} to the {.path .Renviron} file.", cwb_dirs[["registry_dir"]]),
+        collapse = " "
+      ),
+      wrap = TRUE
+    )
+    cat("\n")
+    answer <- menu(
+      choices = c("Yes", "No"), 
+      title = cli_text("Do you want to set the {.envvar CORPUS_REGISTRY} environment variable in the {.path .Renviron}-file now?")
+    )
+    if (answer == 1){
+      renviron_file <- use_corpus_registry_envvar(registry_dir = cwb_dirs[["registry_dir"]])
+    } else {
+      cli_alert_warning(
+        paste(
+          "The {.path .Renviron} file will not be changed.",
+          "Remember to set the {.envvar CORPUS_REGISTRY} environment variable temporarily ",
+          sprintf('by calling {.code Sys.getenv(CORPUS_REGISTRY = "%s")}', cwb_dirs[["registry_dir"]]),
+          "or permanently by defining it in the {.path .Renviron} file!"
+        ),
+        wrap = TRUE
+      )
+    }
+    
+  }
+  invisible(TRUE)
 }
 
 #' @details \code{corpus_packages} will detect the packages that include CWB
@@ -393,8 +438,9 @@ corpus_rename <- function(old, new, registry_dir = Sys.getenv("CORPUS_REGISTRY")
 
 #' @param ask A \code{logical} value, whether to ask user for confirmation
 #'   before removing a corpus.
-#' @details \code{corpus_remove} can be used to drop a corpus.
+#' @details \code{corpus_remove} can be used to delete a corpus.
 #' @rdname corpus_utils
+#' @importFrom cli cli_alert_success
 #' @export corpus_remove
 corpus_remove <- function(corpus, registry_dir = cwb_registry_dir(), ask = interactive()){
   
@@ -404,15 +450,16 @@ corpus_remove <- function(corpus, registry_dir = cwb_registry_dir(), ask = inter
   data_directory <- reg[["home"]]
   if (ask){
     userinput <- menu(
-      choices = c("Yes / continue", "No / abort"),
-      title = sprintf("Are you sure you want to delete registry and data files for corpus '%s'?", corpus)
+      choices = c("Yes", "No"),
+      title = sprintf("Are you sure you want to delete registry and data files for corpus '%s'?", cli::col_red(corpus))
     )
     if (userinput != 1L) stop("Aborting")
   }
   for (x in list.files(data_directory, full.names = TRUE)) file.remove(x)
   file.remove(data_directory)
   file.remove(file.path(registry_dir, tolower(corpus), fsep = "/"))
-  TRUE
+  cli_alert_success("corpus has been removed (registry and data files)")
+  invisible(TRUE)
 }
 
 #' @details \code{corpus_as_tarball} will create a tarball (.tar.gz-file) with
@@ -475,6 +522,7 @@ corpus_as_tarball <- function(corpus, registry_dir, data_dir, tarfile, verbose =
 #' @param progress Logical, whether to show a progress bar.
 #' @details \code{corpus_copy} will create a copy of a corpus (useful for
 #'   experimental modifications, for instance).
+#' @importFrom cli make_spinner ansi_with_hidden_cursor cli_alert_success
 #' @export corpus_copy
 #' @rdname corpus_utils
 #' @examples
@@ -513,23 +561,56 @@ corpus_copy <- function(
   if (!dir.exists(registry_dir_new)) dir.create(registry_dir_new, recursive = TRUE)
   if (!dir.exists(data_dir_new)) dir.create(data_dir_new, recursive = TRUE)
 
-  if (verbose) message(sprintf("... creating copy of adjusted registry file"))
-  rf <- registry_file_parse(corpus = corpus, registry_dir = registry_dir)
-  rf[["home"]] <- data_dir_new
-  registry_file_write(rf, corpus = corpus, registry_dir = registry_dir_new)
-  
-  if (verbose) message(sprintf("... copying data files"))
-  data_files_to_copy <- list.files(data_dir, full.names = TRUE)
-  if (progress) pb <- txtProgressBar(min = 1L, max = length(data_files_to_copy), style = 3)
-  for (i in 1L:length(data_files_to_copy)){
-    if (progress) setTxtProgressBar(pb, value = i)
-    file.copy(
-      from = data_files_to_copy[i],
-      to = file.path(data_dir_new, basename(data_files_to_copy[i]), fsep = "/")
-      )
+  spinner <- make_spinner(template = "{spin} copy corpus data files from temporary directory to target data directory")
+  copy_with_spinner <- function(){
+    lapply(
+      list.files(data_dir, full.names = TRUE),
+      function(f){
+        spinner$spin()
+        file.copy(from = f, to = file.path(data_dir_new, basename(f), fsep = "/"))
+      }
+    )
+    spinner$finish()
   }
-  if (progress) close(pb)
-  invisible(NULL)
+  ansi_with_hidden_cursor(copy_with_spinner())
+  if (verbose) cli_alert_success("copy corpus data files from temporary directory to target data directory ... done")
+
+  if (verbose) cli_process_start("parse registry file")
+  rf <- registry_file_parse(corpus = corpus, registry_dir = registry_dir)
+  if (verbose) cli_process_done()
+
+  # It is a common mistake that the info file is not stated correctly in the registry file,
+  # so this is a forgiving solution to remedy errors
+  info_file_old <- file.path(data_dir, basename(rf[["info"]]))
+  if (!file.exists(info_file_old)){
+    info_file_guessed <- grep(
+      "^.*?/(|\\.)info(\\.md|)$",
+      list.files(data_dir, full.names = TRUE),
+      value = TRUE
+    )
+    if (length(info_file_guessed) > 0L){
+      if (verbose){
+        cli_alert_info(
+          paste(
+            "info file stated in registry does not exist,",
+            "using the following file which likely to be an info file:",
+            sprintf("{.path %s}", info_file_guessed[1])
+          )
+        )
+      }
+      rf[["info"]] <- info_file_guessed[1]
+    } else {
+      cli_alert_danger("no info file found")
+    }
+  }
+  
+  if (verbose) cli_process_start("update registry data and save registry file")
+  rf[["home"]] <- data_dir_new
+  rf[["info"]] <- file.path(data_dir_new, basename(rf[["info"]]))
+  registry_file_write(rf, corpus = corpus, registry_dir = registry_dir_new)
+  if (verbose) cli_process_done()
+
+  invisible(TRUE)
 }
 
 
@@ -648,3 +729,51 @@ corpus_recode <- function(corpus, registry_dir = Sys.getenv("CORPUS_REGISTRY"),
   
   invisible(NULL)
 }
+
+
+#' @importFrom cli cli_process_start
+#' @export corpus_testload
+#' @rdname corpus_utils
+corpus_testload <- function(corpus, registry_dir = Sys.getenv("CORPUS_REGISTRY"), verbose = TRUE){
+  
+  msg <- sprintf("check whether corpus %s can be loaded", corpus)
+  cli_process_start(msg, msg_done = paste(msg, "... OK"))
+  
+  # Initializing CQP is a workaround to address an odd issue that cqp_get_registry()
+  # will not work as expected if cqp_initializ() has not been called before,
+  # see https://github.com/PolMine/RcppCWB/issues/14
+  if (isFALSE(RcppCWB::cqp_is_initialized())){
+    RcppCWB::cqp_initialize(registry = Sys.getenv("CORPUS_REGISTRY"))
+  }
+  
+  if (isFALSE(identical(RcppCWB::cqp_get_registry(), registry_dir))){
+    registry_to_restore <- RcppCWB::cqp_get_registry()
+    RcppCWB::cqp_reset_registry(registry = registry_dir)
+  } else {
+    registry_to_restore <- NA
+  }
+  
+  n <- RcppCWB::cl_attribute_size(
+    corpus = corpus,
+    attribute = "word",
+    attribute_type = "p",
+    registry = registry_dir
+  )
+  
+  if (n > 0L){
+    cli::cli_process_done()
+    retval <- TRUE
+  } else {
+    cli::cli_process_failed()
+    retval <- FALSE
+  }
+  
+  if (isFALSE(is.na(registry_to_restore))){
+    # Check whether dir exists because RcppCWB::cqp_get_registry() is buggy
+    if (dir.exists(registry_to_restore)){
+      RcppCWB::cqp_reset_registry(registry = registry_to_restore)
+    }
+  }
+  
+  invisible(retval)
+}
